import logging
import Levenshtein

log = logging.getLogger(__name__)


class Truth:
    def __init__(self, config):
        self.plates = []
        self.platestmp = []
        self.config = config
        if config.plates_truth:
            self.load()

    def load(self):
        with open(self.config.plates_truth) as f:
            for row in f.readlines():
                platestr = row.strip()
                if platestr:
                    self.plates.append(dict(plate=platestr, distance=-1, detected=""))
            log.info(
                "truth plates loaded %d plates from %s",
                len(self.plates),
                self.config.plates_truth,
            )

    def add(self, plate_new: str):
        min_distance = 999
        best = None
        for plate in self.plates:
            if plate["distance"] == 0:
                continue  # già trovata
            distance = Levenshtein.distance(plate_new, plate["plate"])
            # print(plate_new['plate'], plate['plate'], distance)
            if distance < min_distance:
                best = plate
                min_distance = distance

        if best:
            best["distance"] = min_distance
            best["detected"] = plate_new
        try:
            log.info(
                "truth score: %.1f%%",
                100.0
                * len([p for p in self.plates if p["distance"] == 0])
                / (len(self.plates)),
            )
            log.debug(
                "not matched %d: %s...",
                len(self.plates),
                [p["plate"] for p in self.plates if p["distance"] != 0][:3],
            )
        except Exception:
            pass
        self.platestmp.append(plate_new)
        self.save_results()

    def save_results(self):
        if self.config.plates_truth:
            base, ext = self.config.plates_truth.split(".")
            filename = base + "-processed." + ext

            log.info("truth plates saved to %s", filename)
            with open(filename, "w") as f:
                for plate in self.plates:
                    f.write(
                        f"{plate['plate']},{plate['distance']},{plate['detected']}\n"
                    )

            filename_list = base + "-tmp." + ext
            with open(filename_list, "w") as f:
                for plate_ascii in self.platestmp:
                    f.write(f"{plate_ascii}\n")
