import base64
import os
import random
import sys

import cv2 as cv
import xmltodict


def getBase64(frame, size=(1024, 512), for_http=True):
    if size:
        frame = cv.resize(frame, tuple(size))
    _, buffer = cv.imencode(".jpg", frame)
    jpg_as_text = base64.b64encode(buffer)
    if not for_http:
        return jpg_as_text.decode()
    return "data:image/jpg;base64," + jpg_as_text.decode()


def toBinary(frame):
    return cv.imencode(".jpg", frame)[1].tobytes()


# def sendSignalToProcesses(name, sig="SIGUSR1"):

#     _sig = getattr(signal, sig)

#     for proc in psutil.process_iter():
#         if proc.name() == name:
#             if proc.parent().name() != name:
#                 print(f"send signal {_sig} to {proc.name()}")
#                 proc.send_signal(_sig)


def random_crop(img, width, height):
    assert img.shape[0] >= height
    assert img.shape[1] >= width

    x = random.randint(0, img.shape[1] - width)
    y = random.randint(0, img.shape[0] - height)
    img = img[y : y + height, x : x + width]

    return img


# def projection(img, show=False):
#     rows, cols = img.shape[:2]
#     h_projection = np.array([x / rows for x in img.sum(axis=0)])

#     if show:
#         fig = plt.figure(figsize=(16, 8))
#         fig.add_subplot(121)
#         for j in h_projection:
#             # img[:, j] = 0
#             plt.plot((j, j), (0, 1), "g-")
#         plt.plot(range(cols), h_projection.T)
#         plt.show()


def rotate(img):
    out = cv.transpose(img)
    return cv.flip(out, flipCode=1)


def add_overlay(img, crop, label="", label2="", fontSize=0.8, tickness=2):
    """
    add crop image over img image
    """
    crop = cv.copyMakeBorder(src=crop, top=2, bottom=2, left=2, right=2, borderType=cv.BORDER_CONSTANT)
    h, w = crop.shape[:2]
    H, W = img.shape[:2]
    # font = cv.FONT_HERSHEY_SIMPLEX
    font = cv.FONT_HERSHEY_PLAIN
    if h < H and w < W:
        img[0:h, 0:w, :] = crop
    if label:
        labelSize, baseLine = cv.getTextSize(label, font, fontSize, tickness)
        left = 0
        top = h + labelSize[1]
        try:
            cv.rectangle(
                img,
                (left, top - labelSize[1]),
                (left + w, top + baseLine),
                (255, 255, 255),
                cv.FILLED,
            )
            cv.putText(
                img,
                label,
                (left + 3, top + int(labelSize[1] / 4)),
                font,
                fontSize,
                (0, 0, 0),
                tickness,
                cv.LINE_AA,
            )
            if label2:
                top = top + labelSize[1] + 5
                cv.rectangle(
                    img,
                    (left, top - labelSize[1]),
                    (left + w, top + baseLine),
                    (255, 255, 255),
                    cv.FILLED,
                )
                cv.putText(
                    img,
                    label2,
                    (left + 3, top + 3),
                    font,
                    fontSize,
                    (0, 0, 0),
                    tickness,
                    cv.LINE_AA,
                )
        except Exception:
            pass
    return img


def get_bboxes(xmlfile, classes=[]):
    if not os.path.exists(xmlfile):
        return []
    data = xmltodict.parse(open(xmlfile).read(), force_list=["object"])["annotation"]
    bboxes = []
    for obj in data.get("object", []):
        xmax = int(float(obj["bndbox"]["xmax"]))
        xmin = int(float(obj["bndbox"]["xmin"]))
        ymax = int(float(obj["bndbox"]["ymax"]))
        ymin = int(float(obj["bndbox"]["ymin"]))
        if classes:
            try:
                classId = classes.index(obj["name"])
            except Exception:
                print(
                    "error in %s: '%s' not in dataset" % (xmlfile, obj["name"]),
                    file=sys.stderr,
                )
                classId = 0
                # sys.exit(1)
        else:
            classId = 0
        bboxes.append(
            {
                "classId": classId,
                "confidence": 1.0,
                "box": (xmin, ymin, xmax - xmin + 1, ymax - ymin + 1),
                "name": obj["name"],
            }
        )
    return bboxes


def get_bboxes_txt(txtfile, w, h, classes=[]):
    if not os.path.exists(txtfile):
        return []
    data = open(txtfile).readlines()
    bboxes = []
    for obj in data:
        classId, xc, yc, wf, hf = map(float, obj.strip().split(" "))

        classId = int(classId)
        xmax = int(round((wf / 2.0 + xc) * float(w)))
        xmin = int(round(max(0, (xc - wf / 2.0) * float(w))))
        ymax = int(round((hf / 2.0 + yc) * float(h)))
        ymin = int(round(max(0, (yc - hf / 2.0) * float(h))))
        bboxes.append(
            {
                "classId": classId,
                "confidence": 1.0,
                "box": (xmin, ymin, xmax - xmin + 1, ymax - ymin + 1),
                "name": classes[classId],
            }
        )
    return bboxes
